# frozen_string_literal: true

require "rake"
require "mina"
require "pp"
require "open3"

class VersionError < StandardError; end

module Groundskeeper
  # Formulas for managing releases and deployments.
  # rubocop:disable Metrics/ClassLength
  class Commands
    RAKEFILE = File.join(
      File.dirname(__FILE__), "..", "..", "config", "deploy.rb"
    )
    STAGE = "stage"
    STAGING = "staging"
    PRODUCTION = "production"
    DEFAULT_STAGE = STAGING
    TAG = "tag"
    INITIAL_VERSION = "0.0.1"
    SSH_USERNAME = "deploy"

    # rubocop:disable Metrics/AbcSize,Metrics/MethodLength
    def self.build(console)
      repository = Repository.new
      project = Project.build(repository.name)
      deploy_url = "https://#{project.full_dns(stage)}"
      website = Website.new(deploy_url)
      bitbucket = Bitbucket.build
      sentry = Sentry.build(
        project_name: project.sentry_project,
        version_prefix: repository.name
      )
      slack = Slack.build
      ssh = Ssh.build(SSH_USERNAME, project.full_dns(stage))

      new(
        changelog: Changelog.build,
        console: console,
        bitbucket: bitbucket,
        git: Git.build,
        jira: Jira.build(project.jira_prefix),
        project: project,
        repository: repository,
        rubygems: Rubygems,
        sentry: sentry,
        slack: slack,
        ssh: ssh,
        version_file: RailsVersion.build,
        website: website
      )
    end
    # rubocop:enable Metrics/AbcSize,Metrics/MethodLength

    def self.stage
      ENV[STAGE] == PRODUCTION ? PRODUCTION : DEFAULT_STAGE
    end

    # rubocop:disable Metrics/MethodLength,Metrics/ParameterLists
    def initialize(
      changelog: nil,
      console:,
      bitbucket: nil,
      git: nil,
      jira: nil,
      project: nil,
      repository: nil,
      rubygems: nil,
      sentry: nil,
      slack: nil,
      ssh: nil,
      website: nil,
      version_file:
    )
      @changelog = changelog
      @console = console
      @bitbucket = bitbucket
      @git = git
      @jira = jira
      @project = project
      @repository = repository
      @rubygems = rubygems
      @sentry = sentry
      @slack = slack
      @ssh = ssh
      @website = website
      @version_file = version_file
      @did_push_to_remote = false
      @current_step = 1
    end
    # rubocop:enable Metrics/MethodLength,Metrics/ParameterLists

    # rubocop:disable Metrics/MethodLength,Metrics/AbcSize
    def info(options = {})
      return unrecognized_version unless version_file.exists?
      return groundskeeper_outdated unless check_groundskeeper_version

      Executable.verbose = options[:verbose]
      pull_project_details

      announce_latest_tag
      console.say(
        "version in current branch: " +
        version_file.current_version,
        :yellow
      )
      console.say(
        "tag contained in: #{git.branches_containing_latest_tag}\n\n",
        :yellow
      )
    end
    # rubocop:enable Metrics/AbcSize

    # rubocop:disable Metrics/AbcSize, Metrics/CyclomaticComplexity,
    def release(options = {})
      is_initial_release = !version_file.exists?

      return missing_jira_credentials unless jira.credentials?
      return missing_bitbucket_credentials unless bitbucket.credentials?
      return groundskeeper_outdated unless check_groundskeeper_version
      return unrecognized_version unless version_file.rails?

      version_file.create_initial_version! unless version_file.exists?
      Executable.verbose = options[:verbose]
      pull_project_details

      summarize_recent_commits unless is_initial_release
      say_next_version(is_initial_release)
      checkout_new_branch
      update_version_file
      update_changelog
      commit_changes_and_tag(is_initial_release)
      create_jira_version
      push
      add_version_to_jira_issues unless is_initial_release
      create_pr
    end
    # rubocop:enable Metrics/AbcSize, Metrics/CyclomaticComplexity
    # rubocop:enable Metrics/MethodLength

    # :nocov:
    def predeploy(options = {})
      return unrecognized_version unless version_file.exists?
      return groundskeeper_outdated unless check_groundskeeper_version

      Executable.verbose = options[:verbose]
      pull_project_details

      mina "predeploy", options
    end
    # :nocov:

    # rubocop:disable Metrics/AbcSize, Metrics/CyclomaticComplexity
    # rubocop:disable Metrics/MethodLength, Metrics/PerceivedComplexity
    def deploy(options = {})
      return unrecognized_version unless version_file.exists?
      return unrecognized_tag unless tag_present_in_git?(ENV[TAG])
      return missing_jira_credentials unless jira.credentials?
      return missing_sentry_credentials unless sentry.credentials?
      return missing_slack_credentials unless slack.credentials?
      return groundskeeper_outdated unless check_groundskeeper_version
      return unable_to_ssh unless ssh.can_connect?

      Executable.verbose = options[:verbose]
      pull_project_details
      ENV["whenever"] = "1" if project.uses_whenever?

      mina "deploy", options
      announce_step "Wait for deployed application to restart"
      update_deployed_issues
      release_version if self.class.stage == PRODUCTION
      add_version_to_sentry
    end
    # rubocop:enable Metrics/AbcSize, Metrics/CyclomaticComplexity
    # rubocop:enable Metrics/MethodLength, Metrics/PerceivedComplexity

    private

    # collaborators
    attr_reader :bitbucket, :changelog, :console, :git, :jira, :project,
                :repository, :rubygems, :sentry, :slack, :ssh, :version_file,
                :website
    # state
    attr_reader :current_step, :did_push_to_remote, :next_version,
                :recent_commits

    def mina(command, options)
      announce_step "Run mina to deploy"
      cmd = String.new command
      cmd << " -s" if options[:simulate]
      cmd << " -v" if options[:verbose]
      cmd << " force_asset_precompile=true" if options[:force_asset_precompile]
      cmd << " nginx=true" if options[:nginx]
      run_mina cmd
    end

    def pull_project_details
      @pull_project_details ||= begin
        announce_step("Updating ~/.project_details from "\
                      "#{git.remote_url('.project_details')}")
        git.pull(".project_details")
      end
    end

    def check_groundskeeper_version
      @check_groundskeeper_version ||= begin
        console.say("Groundskeeper version #{Groundskeeper::VERSION}\n\n",
                    :bold)
        latest_version = rubygems.latest_groundskeeper_version

        return true unless SemanticVersion.new(latest_version) > VERSION

        return false unless install_latest_groundskeeper?

        console.run("gem install groundskeeper-bitcore")

        true
      end
    end

    # :nocov:
    def install_latest_groundskeeper?
      question = <<~TEXT.squish
        Groundskeeper is outdated,
        would you like the latest version installed?
      TEXT

      console.ask(question, :red, limited_to: %w[yes no]) == "yes"
    end
    # :nocov:

    def announce_latest_tag
      console.say(
        "latest tag on this repository: " +
        git.latest_tag_name_across_branches,
        :yellow
      )
    end

    def summarize_recent_commits
      console.say("commits since last tag", :bold)
      @recent_commits = repository.changes_since_latest_tag
      console.say(recent_commits.join("\n"), :green)
      console.say("")
    end

    def say_next_version(is_initial_release)
      if is_initial_release
        @next_version = INITIAL_VERSION
      else
        type = "m"
        @next_version = repository.bumped_semantic_version(type)
      end

      console.say("next tag will be: #{next_version}", :green)
    end

    def checkout_new_branch
      git.create_and_checkout_branch(new_branch_name)
    end

    def update_version_file
      version_file.update_version! next_version
      console.say("# updated version file", :green)
    end

    def update_changelog
      changelog.update_file(next_version, recent_commits)
      console.say("# updated changelog", :green)
    end

    def commit_changes_and_tag(is_initial_release)
      is_initial_release ? git.add_all : git.add_update
      git.commit(format(Repository::RELEASE_MESSAGE, next_version))
      console.say("# committed changes", :green)
    end

    def create_jira_version
      jira.create_remote_version(next_jira_version_name)
    end

    def push
      @did_push_to_remote = true
      git.push && console.say("# pushed to version control")
    end

    def add_version_to_jira_issues
      issue_ids = jira.included_issues(recent_commits)
      message = jira.add_version_to_remote_issues(
        next_jira_version_name, issue_ids
      )
      if message.blank?
        console.say("# added version to Jira issues #{issue_ids}", :green)
      else
        announce_error(message)
      end
    end

    def create_pr
      bitbucket.create_pr(
        repository: repository.name, branch_name: new_branch_name
      )
    end

    def next_jira_version_name
      "#{repository.name} #{next_version}"
    end

    def add_version_to_sentry
      announce_step "Add release to Sentry and mark deployed"
      sentry.create_release(ENV[TAG])
      sentry.associate_commits(ENV[TAG])
      sentry.deploy(ENV[TAG], self.class.stage)
      console.say("# added version to Sentry", :green)
    end

    def tag_present_in_git?(tag)
      tag.nil? ? false : git.list_tags.include?(tag)
    end

    def announce_error(message)
      console.say("# #{message}", :red)
    end

    def unrecognized_tag
      console.say(
        "Tag not found, try again with an existing tag.",
        :red
      )
    end

    def unrecognized_version
      console.say(
        "Unable to find version file, is this a Rails application?",
        :red
      )
    end

    def groundskeeper_outdated
      console.say("Groundskeeper outdated. Install the latest version.", :red)
    end

    def missing_bitbucket_credentials
      console.say(
        "Please configure your Bitbucket environment variables.",
        :red
      )
    end

    def missing_jira_credentials
      console.say(
        "Please configure your Jira environment variables.",
        :red
      )
    end

    def missing_sentry_credentials
      console.say(
        "Please configure your Sentry environment variables.",
        :red
      )
    end

    def missing_slack_credentials
      console.say(
        "Please configure your Slack environment variables.",
        :red
      )
    end

    def unable_to_ssh
      console.say(
        "Unable to SSH to the server, are you signed into VPN?",
        :red
      )
    end

    def new_branch_name
      "release/#{next_version}"
    end

    def run_mina(arguments)
      command = "mina #{arguments} -f #{RAKEFILE}"
      Open3.popen3(command) do |_stdout, stderr, _status, _thread|
        # :nocov:
        # rubocop:disable Lint/AssignmentInCondition
        while line = stderr.gets
          puts line
        end
        # rubocop:enable Lint/AssignmentInCondition
        # :nocov:
      end
    end

    # rubocop:disable Metrics/MethodLength
    def update_deployed_issues
      tries = 0

      announce_step(
        "Check deployed version at #{website.uri}/#{Website::VERSION_PATH}"
      )

      begin
        deployed_version = website.version

        raise VersionError unless deployed_version == ENV[TAG]

        console.say("# deployment successful", :green)
        transition_remote_issues deployed_version
        announce_in_slack deployed_version
      rescue VersionError
        tries += 1
        sleep 5
        retry if tries < 3
        # :nocov:
        console.say(
          "something went wrong: expected version #{ENV[TAG]}, " \
          "found version #{deployed_version} instead",
          :red
        )
        # :nocov:
      end
    end
    # rubocop:enable Metrics/MethodLength

    # :nocov:
    def release_version
      jira.release_version(project, website.version)
      console.say("#{project.repo_name} #{website.version} released", :green)
    end
    # :nocov:

    # rubocop:disable Metrics/MethodLength
    def transition_remote_issues(version)
      version_name = "#{repository.name} #{version}"
      deployed_issues = jira.fetch_issues_by_fix_version(version_name)
      action = if self.class.stage == PRODUCTION
                 Jira::DEPLOY_TO_PRODUCTION
               else
                 Jira::DEPLOY_TO_STAGING
               end
      announce_step(
        "Transitioning deployed issues in Jira: " \
        "#{action} #{deployed_issues.join(', ')}"
      )
      jira.transition_remote_issues(action, deployed_issues)
    end
    # rubocop:enable Metrics/MethodLength

    def announce_in_slack(deployed_version)
      slack.send_message(
        "#{project.project_name} version #{deployed_version} deployed to " +
        self.class.stage
      )
    end

    def announce_step(message)
      console.say(">>>", :magenta)
      console.say(">>> #{@current_step}. #{message}", :magenta)
      console.say(">>>", :magenta)
      console.say("")
      @current_step += 1
    end
  end
  # rubocop:enable Metrics/ClassLength
end
